@@ -5,7 +5,7 @@ import re
 import sys
 from pathlib import Path
 from textwrap import dedent
-from typing import NamedTuple
+from typing import NamedTuple, Union
 from zipfile import ZipFile
 
 from hookman.exceptions import (
@@ -13,35 +13,41 @@ from hookman.exceptions import (
 from hookman.hooks import HookSpecs
 from hookman.plugin_config import PluginInfo
 
-INDENTATION = "    "
-NEW_LINE = "\n"
-
 
 class Hook(NamedTuple):
     """
     Class to assist on the process to generate the files
 
-    name: Name of the Hook
-    macro_name: Name of the Hook in Upper Case
-    r_type: Type of the return from the hook
-
     args: The name of each argument
         Ex.: v1, v2, v3
+
     args_type: The type of each argument
         Ex.: int, float, int
+
     args_with_type: The name of the argument with the type
         Ex.: int v1, float v2, int v3
 
+    documentation: The docstring content from the hook definition
+
     function_name: Full name of the hook function
         Ex.: project_name_version_hook_name -> alfasim_v4_friction_factory
+
+    macro_name: Name of the Hook in Upper Case
+
+    name: Name of the Hook
+
+    r_type: Type of the return from the hook
+
+
     """
-    name: str
-    macro_name: str
-    r_type: str
     args: str
     args_type: str
     args_with_type: str
+    documentation: str
     function_name: str
+    macro_name: str
+    name: str
+    r_type: str
 
 
 class HookManGenerator:
@@ -49,12 +55,13 @@ class HookManGenerator:
     Class to assist in the process of creating necessary files for the hookman
     """
 
-    def __init__(self, hook_spec_file_path: Path) -> None:
+    def __init__(self, hook_spec_file_path: Union[Path, str]) -> None:
         """
         Receives a path to a hooks specification file.
         if the Path provided is not a file an exception FileNotFoundError is raised.
         If the File provided doesn't have a spec object, a RuntimeError is raised.
         """
+        hook_spec_file_path = Path(hook_spec_file_path)
         if hook_spec_file_path.is_file():
             hook_spec_module = self._import_hook_specification_file(hook_spec_file_path)
             self._populate_local_variables(hook_spec_module.specs)
@@ -103,20 +110,23 @@ class HookManGenerator:
                 return f"{array_type} {arg}[{array_size}]"
             return f"{arg_type.strip()} {arg}"
 
+        self.extra_includes = hook_specs.extra_includes
         self.hooks = []
         for hook_spec in hook_specs.hooks:
+            hook_documentation = inspect.getdoc(hook_spec)
             hook_arg_spec = inspect.getfullargspec(hook_spec)
             hook_arguments = hook_arg_spec.args
             hook_types = hook_arg_spec.annotations
             self.hooks.append(
                 Hook(
-                    name=hook_spec.__name__.lower(),
-                    macro_name=hook_spec.__name__.upper(),
-                    r_type=hook_arg_spec.annotations['return'],
                     args=', '.join(hook_arguments),
                     args_type=', '.join([f"{hook_types[arg]}" for arg in hook_arguments]),
                     args_with_type=', '.join(get_arg_with_type(arg) for arg in hook_arguments),
+                    documentation=hook_documentation,
                     function_name=f'{self.project_name}_{self.version}_{hook_spec.__name__.lower()}',
+                    macro_name=hook_spec.__name__.upper(),
+                    name=hook_spec.__name__.lower(),
+                    r_type=hook_arg_spec.annotations['return'],
                 ))
 
     def generate_plugin_template(self,
@@ -128,15 +138,22 @@ class HookManGenerator:
         """
         Generate a template with the necessary files and structure to create a plugin
 
+        :param str plugin_name: the user-friendly name of the plugin, for example ``"Hydrates"``.
+
         A folder with the same name as the shared_lib_name argument will be created, with the following files:
-            - config.yml
-            - plugin.c
-            - hook_specs.h
-            - CMakeLists.txt
-            - README.md
+            <plugin_folder>
+                - CMakeLists.txt
+                - compile.py
+                assets/
+                    - plugin.yaml
+                    - README.md
+                src/
+                    - hook_specs.h
+                    - plugin.c
+                    - CMakeLists.txt
         """
         if not shared_lib_name.isidentifier():
-            raise HookmanError("The shared libray name must be a valid identifier.")
+            raise HookmanError("The shared library name must be a valid identifier.")
 
         plugin_folder = dst_path / shared_lib_name
         assets_folder = plugin_folder / 'assets'
@@ -155,32 +172,39 @@ class HookManGenerator:
         Path(plugin_folder / 'CMakeLists.txt').write_text(self._plugin_cmake_file_content(shared_lib_name))
         Path(assets_folder / 'plugin.yaml').write_text(self._plugin_config_file_content(plugin_name, shared_lib_name, author_email, author_name))
         Path(assets_folder / 'README.md').write_text(self._readme_content(plugin_name, author_email, author_name))
-        Path(source_folder / 'hook_specs.h').write_text(self._hook_specs_header_content())
+        Path(source_folder / 'hook_specs.h').write_text(self._hook_specs_header_content(shared_lib_name))
         Path(source_folder / 'plugin.c').write_text(self._plugin_source_content())
         Path(source_folder / 'CMakeLists.txt').write_text(self._plugin_src_cmake_file_content(shared_lib_name))
 
-    def generate_project_files(self, dst_path: Path):
+    def generate_hook_specs_header(self, shared_lib_name: str, dst_path: Union[str, Path]):
+        """Generates the "hook_specs.h" file which is consumed by plugins to implement the hooks.
+
+        :param shared_lib_name: short name of the generated shared library
+        :param dst_path: directory where to generate the file.
+        """
+        source_folder = Path(dst_path) / shared_lib_name / 'src'
+        source_folder.mkdir(parents=True, exist_ok=True)
+        Path(source_folder / 'hook_specs.h').write_text(self._hook_specs_header_content(shared_lib_name))
+
+
+    def generate_project_files(self, dst_path: Union[Path, str]):
         """
         Generate the following files on the dst_path:
-            - hook_specs.h
-            - HookCaller.hpp
-            - HookCallerPython.cpp
+        - HookCaller.hpp
+        - HookCallerPython.cpp
         """
-        hook_specs_h = Path(dst_path / 'plugin' / 'hook_specs.h')
-        hook_caller_hpp = Path(dst_path / 'cpp' / 'HookCaller.hpp')
-        hook_caller_python = Path(dst_path / 'binding' / 'HookCallerPython.cpp')
-
-        os.makedirs(hook_specs_h.parent)
-        os.makedirs(hook_caller_hpp.parent)
-        os.makedirs(hook_caller_python.parent)
-
-        hook_specs_h.write_text(self._hook_specs_header_content())
+        hook_caller_hpp = Path(dst_path) / 'cpp' / 'HookCaller.hpp'
+        hook_caller_hpp.parent.mkdir(exist_ok=True, parents=True)
         hook_caller_hpp.write_text(self._hook_caller_hpp_content())
-        hook_caller_python.write_text(self._hook_caller_python_content())
+
+        if self.pyd_name:
+            hook_caller_python = Path(dst_path / 'binding' / 'HookCallerPython.cpp')
+            hook_caller_python.parent.mkdir(exist_ok=True, parents=True)
+            hook_caller_python.write_text(self._hook_caller_python_content())
 
         self._generate_cmake_files(dst_path)
 
-    def generate_plugin_package(self, package_name: str, plugin_dir: Path, dst: Path=None):
+    def generate_plugin_package(self, package_name: str, plugin_dir: Union[Path, str], dst_path: Path=None):
         """
         Creates a .hmplugin file using the name provided on package_name argument.
         The file `.hmplugin` will be created with the content from the folder assets and artifacts.
@@ -193,21 +217,23 @@ class HookManGenerator:
         Per default, the package will be created inside the folder plugin_dir, however it's possible
         to give another path filling the dst argument
         """
-        if dst is None:
-            dst = plugin_dir
+        plugin_dir = Path(plugin_dir)
+        if dst_path is None:
+            dst_path = plugin_dir
 
         assets_dir = plugin_dir / "assets"
         artifacts_dir = plugin_dir / "artifacts"
+        python_dir = plugin_dir / "src" / "python"
 
         self._validate_package_folder(artifacts_dir, assets_dir)
         self._validate_plugin_config_file(assets_dir / 'plugin.yaml', artifacts_dir)
 
         if sys.platform == 'win32':
             shared_lib = '*.dll'
-            hmplugin_path = dst / f"{package_name}-win64.hmplugin"
+            hmplugin_path = dst_path / f"{package_name}-win64.hmplugin"
         else:
             shared_lib = '*.so'
-            hmplugin_path = dst / f"{package_name}-linux64.hmplugin"
+            hmplugin_path = dst_path / f"{package_name}-linux64.hmplugin"
 
         with ZipFile(hmplugin_path, 'w') as zip_file:
             for file in assets_dir.rglob('*'):
@@ -216,6 +242,10 @@ class HookManGenerator:
             for file in artifacts_dir.rglob(shared_lib):
                 zip_file.write(filename=file, arcname=file.relative_to(plugin_dir))
 
+            for file in python_dir.rglob('*'):
+                dst_filename = Path('artifacts' / file.relative_to(plugin_dir / 'src/python'))
+                zip_file.write(filename=file, arcname=dst_filename)
+
     def _validate_package_folder(self, artifacts_dir, assets_dir):
         """
         Method to ensure that the plugin folder has the following criteria:
@@ -235,7 +265,7 @@ class HookManGenerator:
             raise AssetsDirNotFoundError()
 
         if not artifacts_dir.exists():
-            raise ArtifactsDirNotFoundError()
+            raise ArtifactsDirNotFoundError(f'Artifacts directory not found: {artifacts_dir}')
 
         shared_lib = '*.dll' if sys.platform == 'win32' else '*.so'
         if not any(artifacts_dir.rglob(shared_lib)):
@@ -260,142 +290,212 @@ class HookManGenerator:
                 f"{plugin_file_content.shared_lib_name} could not be found in {artifacts_dir}"
             )
 
-    def _hook_specs_header_content(self) -> str:
+    def _hook_specs_header_content(self, shared_lib_name) -> str:
         """
         Create a C header file with the content informed on the hook_specs
         """
-        file_content = []
-        list_with_hook_specs_arguments = []
+        list_with_hook_specs_with_documentation = ''
 
         for hook in self.hooks:
-            line = f'#define HOOK_{hook.macro_name}({hook.args}) HOOKMAN_API_EXP {hook.r_type} HOOKMAN_FUNC_EXP {hook.function_name}({hook.args_with_type})' + NEW_LINE
-            list_with_hook_specs_arguments.append(line)
+            hook_specs_content = '\n/*!\n'
+            hook_specs_content += hook.documentation
+            hook_specs_content += dedent(f"""
+            */
+            #define HOOK_{hook.macro_name}({hook.args}) HOOKMAN_API_EXP {hook.r_type} HOOKMAN_FUNC_EXP {hook.function_name}({hook.args_with_type})
+            """)
+            list_with_hook_specs_with_documentation += hook_specs_content
 
-        from textwrap import dedent
-        file_content += dedent(f"""\
+        file_content = dedent(f"""\
+        /* {self._DO_NOT_MODIFY_MSG} */
         #ifndef {self.project_name.upper()}_HOOK_SPECS_HEADER_FILE
         #define {self.project_name.upper()}_HOOK_SPECS_HEADER_FILE
+        #ifdef __cplusplus
+            #define _HOOKMAN_EXTERN_C extern "C"
+        #else
+            #define _HOOKMAN_EXTERN_C
+        #endif
+
         #ifdef WIN32
-            #define HOOKMAN_API_EXP __declspec(dllexport)
+            #define HOOKMAN_API_EXP _HOOKMAN_EXTERN_C __declspec(dllexport)
             #define HOOKMAN_FUNC_EXP __cdecl
         #else
-            #define HOOKMAN_API_EXP
+            #define HOOKMAN_API_EXP _HOOKMAN_EXTERN_C
             #define HOOKMAN_FUNC_EXP
         #endif
 
-        #define INIT_HOOKS() HOOKMAN_API_EXP char* HOOKMAN_FUNC_EXP {self.project_name}_version_api() {{return \"{self.version}\";}}
+        HOOKMAN_API_EXP const char* HOOKMAN_FUNC_EXP {self.project_name}_version_api() {{
+            return \"{self.version}\";
+        }}
+
+        HOOKMAN_API_EXP const char* HOOKMAN_FUNC_EXP get_plugin_name() {{
+            return \"{shared_lib_name}\";
+        }}
+
         """)
-        file_content += list_with_hook_specs_arguments
+        file_content += list_with_hook_specs_with_documentation
         file_content += dedent(f"""
-        #endif
+        
+        #endif // {self.project_name.upper()}_HOOK_SPECS_HEADER_FILE
         """)
-        return ''.join(file_content)
+        return file_content
+
+    _DO_NOT_MODIFY_MSG = 'File automatically generated by hookman, **DO NOT MODIFY MANUALLY**'
 
     def _hook_caller_hpp_content(self) -> str:
         """
         Create a .hpp file with the content informed on the hook_specs
         """
-        file_content = []
+        content_lines = []
         list_with_hook_calls = []
         list_with_set_functions = []
         list_with_private_members = []
-        from textwrap import dedent
 
-        file_content += dedent(f"""\
-        #include <functional>
-
-        namespace hookman {{
-
-        template <typename F_TYPE> std::function<F_TYPE> from_c_pointer(uintptr_t p) {{
-        {INDENTATION}return std::function<F_TYPE>(reinterpret_cast<F_TYPE *>(p));
-        }}
-
-        class HookCaller {{
-        public:
-        """)
-
-        list_with_hook_calls += [
-            f'{INDENTATION}std::function<{hook.r_type}({hook.args_type})> {hook.name}() {{' + NEW_LINE +
-            f'{INDENTATION*2}return this->_{hook.name};' + NEW_LINE +
-            f'{INDENTATION}}}' + NEW_LINE
-            for hook in self.hooks
+        content_lines += [
+            f"// {self._DO_NOT_MODIFY_MSG}",
+            "#ifndef _H_HOOKMAN_HOOK_CALLER",
+            "#define _H_HOOKMAN_HOOK_CALLER",
+            "",
+            "#include <functional>",
+            "#include <stdexcept>",
+            "#include <string>",
+            "#include <vector>",
+            "",
+            "#ifdef _WIN32",
+            f"    #include <windows.h>",
+            "#else",
+            f"    #include <dlfcn.h>",
+            "#endif",
+            "",
         ]
-        list_with_private_members += [
-            f'{INDENTATION}std::function<{hook.r_type}({hook.args_type})> _{hook.name};' + NEW_LINE
-            for hook in self.hooks
+        content_lines += (f'#include <{x}>' for x in self.extra_includes)
+        content_lines += [
+            "",
+            "namespace hookman {",
+            "",
+            "template <typename F_TYPE> std::function<F_TYPE> from_c_pointer(uintptr_t p) {",
+            f"    return std::function<F_TYPE>(reinterpret_cast<F_TYPE *>(p));",
+            "}",
+            "",
+            "class HookCaller {",
+            "public:",
         ]
 
-        list_with_set_functions += [
-            f'{1*INDENTATION}void set_{hook.name}_function(uintptr_t pointer) {{' + NEW_LINE +
-            f'{2*INDENTATION}this->_{hook.name} = from_c_pointer<{hook.r_type}({hook.args_type})>(pointer);' + NEW_LINE +
-            f'{1*INDENTATION}}}' + 2 * NEW_LINE
-            for hook in self.hooks
-        ]
-        file_content += list_with_hook_calls
-        file_content += NEW_LINE
-        file_content += list_with_set_functions
-        file_content += "private:" + NEW_LINE
-        file_content += list_with_private_members
-        file_content += "};" + NEW_LINE + "}" + NEW_LINE
+        for hook in self.hooks:
+            list_with_hook_calls += [
+                f'    std::vector<std::function<{hook.r_type}({hook.args_type})>> {hook.name}_impls() {{',
+                f'        return this->_{hook.name}_impls;',
+                f'    }}',
+            ]
+            list_with_private_members += [
+                f'    std::vector<std::function<{hook.r_type}({hook.args_type})>> _{hook.name}_impls;'
+            ]
 
-        return ''.join(file_content)
+            list_with_set_functions += [
+                # uintptr overload
+                f'    void append_{hook.name}_impl(uintptr_t pointer) {{',
+                f'        this->_{hook.name}_impls.push_back(from_c_pointer<{hook.r_type}({hook.args_type})>(pointer));',
+                f'    }}',
+                "",
+                # std::function overload
+                f'    void append_{hook.name}_impl(std::function<{hook.r_type}({hook.args_type})> func) {{',
+                f'        this->_{hook.name}_impls.push_back(func);',
+                f'    }}',
+            ]
+        content_lines += list_with_hook_calls
+        content_lines.append("")
+        content_lines += list_with_set_functions
+        content_lines.append("")
+        content_lines += _generate_load_function(self.hooks)
+        content_lines.append("private:")
+        content_lines += list_with_private_members
+        content_lines.append("};")
+        content_lines.append("")
+        content_lines.append("}  // namespace hookman")
+        content_lines.append("#endif // _H_HOOKMAN_HOOK_CALLER")
+        content_lines.append('')
+
+        return "\n".join(content_lines)
 
     def _hook_caller_python_content(self) -> str:
         """
         Create a .cpp file to bind python and cpp code with PyBind11
         """
-        file_content = []
-        file_content += dedent(f"""\
-            #include <pybind11/pybind11.h>
-            #include <pybind11/functional.h>
-            #include <HookCaller.hpp>
-
-            namespace py = pybind11;
-
-            PYBIND11_MODULE({self.pyd_name}, m) {{
-                py::class_<hookman::HookCaller>(m, "HookCaller")
-                    .def(py::init<>())
-        """)
-        file_content += [
-            f'{2*INDENTATION}.def("{hook.name}", &hookman::HookCaller::{hook.name})' + NEW_LINE +
-            f'{2*INDENTATION}.def("set_{hook.name}_function", &hookman::HookCaller::set_{hook.name}_function)' + NEW_LINE
-            for hook in self.hooks
+        content_lines = [
+            f"// {self._DO_NOT_MODIFY_MSG}",
+            "#include <pybind11/functional.h>",
+            "#include <pybind11/pybind11.h>",
+            "#include <pybind11/stl_bind.h>",
+            "#include <HookCaller.hpp>",
+            "",
+            "namespace py = pybind11;",
+            "",
         ]
-        file_content += f'{2*INDENTATION};' + NEW_LINE + '}' + NEW_LINE
-        return ''.join(file_content)
+        signatures = set((x.r_type, x.args_type) for x in self.hooks)
+        for r_type, args_type in sorted(signatures):
+            content_lines.append(f"PYBIND11_MAKE_OPAQUE(std::vector<std::function<{r_type}({args_type})>>);")
+        content_lines.append("")
+
+        content_lines.append(f"PYBIND11_MODULE({self.pyd_name}, m) {{")
+
+        for index, (r_type, args_type) in enumerate(sorted(signatures)):
+            name = f'vector_hook_impl_type_{index}'
+            vector_type = f"std::vector<std::function<{r_type}({args_type})>>"
+            content_lines.append(f'    py::bind_vector<{vector_type}>(m, "{name}", "Hook for vector implementation type {index}");')
+        content_lines.append("")
+
+        content_lines += [
+            f'    py::class_<hookman::HookCaller>(m, "HookCaller")',
+            f"        .def(py::init<>())",
+            f'        .def("load_impls_from_library", &hookman::HookCaller::load_impls_from_library)',
+        ]
+        for hook in self.hooks:
+            append_ptr = f'&hookman::HookCaller::append_{hook.name}_impl'
+            append_uint_sig = 'void (hookman::HookCaller::*)(uintptr_t)'
+            append_function_sig = f'void (hookman::HookCaller::*)(std::function<{hook.r_type}({hook.args_type})>)'
+
+            content_lines += [
+                f'        .def("{hook.name}_impls", &hookman::HookCaller::{hook.name}_impls)',
+                f'        .def("append_{hook.name}_impl", ({append_uint_sig}) {append_ptr})',
+                f'        .def("append_{hook.name}_impl", ({append_function_sig}) {append_ptr})',
+            ]
+        content_lines.append(f'    ;')
+        content_lines.append('}')
+        content_lines.append('')
+        return "\n".join(content_lines)
 
     def _generate_cmake_files(self, dst_path: Path):
-        hook_caller_hpp = Path(dst_path / 'cpp' / 'CMakeLists.txt')
-        hook_caller_python = Path(dst_path / 'binding' / 'CMakeLists.txt')
         from textwrap import dedent
 
+        hook_caller_hpp = Path(dst_path / 'cpp' / 'CMakeLists.txt')
         with open(hook_caller_hpp, mode='w') as file:
             file.writelines(dedent(f"""\
                 add_library({self.pyd_name}_interface INTERFACE)
                 target_include_directories({self.pyd_name}_interface INTERFACE ./)
                 """))
 
-        with open(hook_caller_python, mode='w') as file:
-            file.writelines(dedent(f"""\
-            include(pybind11Config)
-
-            pybind11_add_module(
-                {self.pyd_name}
-                    HookCallerPython.cpp
-            )
-            target_include_directories(
-               {self.pyd_name}
-                PRIVATE
-                    ${{pybind11_INCLUDE_DIRS}}  # from pybind11Config
-            )
-            target_link_libraries(
-                {self.pyd_name}
-                PRIVATE
-                    {self.pyd_name}_interface
-            )
-
-            install(TARGETS {self.pyd_name} EXPORT ${{PROJECT_NAME}}_export DESTINATION ${{ARTIFACTS_DIR}})
-            """))
+        if self.pyd_name:
+            hook_caller_python = Path(dst_path / 'binding' / 'CMakeLists.txt')
+            with open(hook_caller_python, mode='w') as file:
+                file.writelines(dedent(f"""\
+                include(pybind11Config)
+    
+                pybind11_add_module(
+                    {self.pyd_name}
+                        HookCallerPython.cpp
+                )
+                target_include_directories(
+                   {self.pyd_name}
+                    PRIVATE
+                        ${{pybind11_INCLUDE_DIRS}}  # from pybind11Config
+                )
+                target_link_libraries(
+                    {self.pyd_name}
+                    PRIVATE
+                        {self.pyd_name}_interface
+                )
+    
+                install(TARGETS {self.pyd_name} EXPORT ${{PROJECT_NAME}}_export DESTINATION ${{ARTIFACTS_DIR}})
+                """))
 
     def _plugin_config_file_content(
             self,
@@ -434,7 +534,7 @@ class HookManGenerator:
         Create a C header file with the content informed on the hook_specs
         """
         file_content = []
-        plugin_hooks_macro = [f'// HOOK_{hook.macro_name}({hook.args}){{}}{NEW_LINE}' for hook in self.hooks]
+        plugin_hooks_macro = [f'// HOOK_{hook.macro_name}({hook.args}){{}}\n' for hook in self.hooks]
 
         file_content += dedent(f"""\
         #include "hook_specs.h"
@@ -511,8 +611,8 @@ class HookManGenerator:
             if artifacts_dir.exists():
                 shutil.rmtree(artifacts_dir)
 
-            subprocess.run(["cmake", binary_directory_path, home_directory_path, "-G", build_generator])
-            subprocess.run(["cmake", "--build", str(build_dir), "--config", "Release", "--target", "install"])
+            subprocess.check_call(["cmake", binary_directory_path, home_directory_path, "-G", build_generator])
+            subprocess.check_call(["cmake", "--build", str(build_dir), "--config", "Release", "--target", "install"])
 
             if package_dir.exists():
                 shutil.rmtree(package_dir)
@@ -521,3 +621,140 @@ class HookManGenerator:
             shutil.copy2(src=shared_lib, dst=package_dir)
         ''')
         return file_content
+
+
+def _generate_load_function(hooks):
+    result = [
+        "#if defined(_WIN32)",
+        "",
+    ]
+    result += _generate_windows_body(hooks)
+    result += [
+        "",
+        "#elif defined(__linux__)",
+        "",
+    ]
+    result += _generate_linux_body(hooks)
+    result += [
+        "",
+        "#else",
+        f'    #error "unknown platform"',
+        "#endif",
+        "",
+    ]
+    return result
+
+
+def _generate_windows_body(hooks):
+    """Generate Windows specific functions.
+
+    At the moment it implements load_impls_from_library, class destructor, and an utility function
+    to convert from utf8 to wide-strings so we can use the wide family of windows
+    functions that accept unicode.
+    """
+    # generate destructor to free the library handles opened by load_from_library()
+    result = [
+        "public:",
+        f"    ~HookCaller() {{",
+        f"        for (auto handle : this->handles) {{",
+        f"            FreeLibrary(handle);",
+        f"        }}",
+        f"    }}",
+    ]
+
+    # generate load_impls_from_library()
+    result += [
+        f"    void load_impls_from_library(const std::string& utf8_filename) {{",
+        f'        std::wstring w_filename = utf8_to_wstring(utf8_filename);',
+        f'        auto handle = LoadLibraryW(w_filename.c_str());',
+        f'        if (handle == NULL) {{',
+        f'            throw std::runtime_error("Error loading library " + utf8_filename + ": " + std::to_string(GetLastError()));',
+        f'        }}',
+        f'        this->handles.push_back(handle);',
+        "",
+    ]
+
+    for index, hook in enumerate(hooks):
+        result += [
+            f'        auto p{index} = GetProcAddress(handle, "{hook.function_name}");',
+            f'        if (p{index} != nullptr) {{',
+            f'            this->append_{hook.name}_impl((uintptr_t)(p{index}));',
+            f'        }}',
+            "",
+        ]
+    result.append("    }")
+
+    result += [
+        "",
+        "",
+        "private:",
+        f"    std::wstring utf8_to_wstring(const std::string& s) {{",
+        f"        int flags = 0;",
+        f"        int required_size = MultiByteToWideChar(CP_UTF8, flags, s.c_str(), -1, nullptr, 0);",
+        f"        std::wstring result;",
+        f"        if (required_size == 0) {{",
+        f"            return result;",
+        f"        }}",
+        f"        result.resize(required_size);",
+        f"        int err = MultiByteToWideChar(CP_UTF8, flags, s.c_str(), -1, &result[0], required_size);",
+        f"        if (err == 0) {{",
+        f"            // error handling: https://docs.microsoft.com/en-us/windows/desktop/api/stringapiset/nf-stringapiset-multibytetowidechar#return-value",
+        f"            switch (GetLastError()) {{",
+        f"                case ERROR_INSUFFICIENT_BUFFER: throw std::runtime_error(\"utf8_to_wstring: ERROR_INSUFFICIENT_BUFFER\");",
+        f"                case ERROR_INVALID_FLAGS: throw std::runtime_error(\"utf8_to_wstring: ERROR_INVALID_FLAGS\");",
+        f"                case ERROR_INVALID_PARAMETER: throw std::runtime_error(\"utf8_to_wstring: ERROR_INVALID_PARAMETER\");",
+        f"                case ERROR_NO_UNICODE_TRANSLATION: throw std::runtime_error(\"utf8_to_wstring: ERROR_NO_UNICODE_TRANSLATION\");",
+        f"                default: throw std::runtime_error(\"Undefined error: \" + std::to_string(GetLastError()));",
+        f"            }}",
+        f"        }}",
+        f"        return result;",
+        f"    }}",
+        f"",
+        f"",
+        f"private:",
+        f"    std::vector<HMODULE> handles;",
+    ]
+    return result
+
+
+def _generate_linux_body(hooks):
+    """
+    Generate linux specific functions.
+
+    At the moment it implements load_impls_from_library and the class destructor
+    to cleanup handles.
+    """
+    # generate destructor to free the library handles opened by load_from_library()
+    result = [
+        f"private:",
+        f"    std::vector<void*> handles;",
+        "",
+        "public:",
+        f"    ~HookCaller() {{",
+        f"        for (auto handle : this->handles) {{",
+        f"            dlclose(handle);",
+        f"        }}",
+        f"    }}",
+    ]
+
+    # generate load_impls_from_library()
+    result += [
+        f"    void load_impls_from_library(const std::string& utf8_filename) {{",
+        f'        auto handle = dlopen(utf8_filename.c_str(), RTLD_LAZY);',
+        f'        if (handle == nullptr) {{',
+        f'            throw std::runtime_error("Error loading library " + utf8_filename + ": dlopen failed");',
+        f'        }}',
+        f'        this->handles.push_back(handle);',
+        "",
+    ]
+
+    for index, hook in enumerate(hooks):
+        result += [
+            f'        auto p{index} = dlsym(handle, "{hook.function_name}");',
+            f'        if (p{index} != nullptr) {{',
+            f'            this->append_{hook.name}_impl((uintptr_t)(p{index}));',
+            f'        }}',
+            "",
+        ]
+    result.append("    }")
+    return result
