@@ -7,6 +7,14 @@ def friction_factor(v1: 'int', v2: 'int') -> 'int':
     """
 
 
+def friction_factor_2(v1: 'int', v2: 'int') -> 'int':
+    """
+    Docs for Friction Factor
+
+    Just to test a duplicated signature
+    """
+
+
 def env_temperature(v3: 'float', v4: 'float') -> 'float':
     """
     Docs for Environment Temperature
@@ -19,6 +27,7 @@ specs = HookSpecs(
     pyd_name='_simple',
     hooks=[
         friction_factor,
+        friction_factor_2,
         env_temperature,
     ]
 )
